from math import ceil, log

def format_number(n, accuracy=6):
    """Formats a number in a friendly manner
    (removes trailing zeros and unneccesary point."""

    fs = "%."+str(accuracy)+"f"
    str_n = fs % float(n)
    if '.' in str_n:
        str_n = str_n.rstrip('0').rstrip('.')
    if str_n == "-0":
        str_n = "0"
    return str_n


def lerp(a, b, i):
    """Linear enterpolate from a to b."""
    return a + (b - a) * i


def range2d(range_x, range_y):

    """Creates a 2D range."""

    range_x = list(range_x)
    return [(x, y) for y in range_y for x in range_x]


def xrange2d(range_x, range_y):

    """Iterates over a 2D range."""

    range_x = list(range_x)
    for y in range_y:
        for x in range_x:
            yield (x, y)


def saturate(value, low, high):
    return min(max(value, low), high)


def is_power_of_2(n):
    """Returns True if a value is a power of 2."""
    return log(n, 2) % 1.0 == 0.0


def next_power_of_2(n):
    """Returns the next power of 2 that is >= n"""
    return int(2 ** ceil(log(n, 2)))

def split_lines(string, max_length=90):
    """
    # Function name: linesplitter
    # Purpose: Splits a single string into a list of strings of a defined maximum length
    # Inputs: string - A string that is to be split up
    #         max_length - maximum length of each line
    # Outputs: lines - The string broken up
    """

    next_word = ""
    lines = [""]
    linecount = 0
    # Iterates through each character in the string
    for i in xrange(0, len(string)):
        char = string[i]
        next_word += char
        # If the character is a space, we consider that a complete word
        # If the length of the word + length of the current line is shorter than the
        # Max length / line then we add it to the line. Otherwise we create a new line
        if char == " " or char == "-" or i == len(string)-1:
            if len(lines[linecount]) + len(next_word) < max_length:
                lines[linecount] += next_word
            else:
                linecount += 1
                lines.append("")
                lines[linecount] += next_word
            next_word = ""

    return lines

def set_transparency(surface, alpha_scaling):
    working_surface = surface.copy()
    working_surface.lock()
    for x in xrange(0, working_surface.get_width() - 1):
        for y in xrange(0, working_surface.get_height() - 1):
            r, g, b, a = working_surface.get_at((x, y))

            if a:
                working_surface.set_at((x, y), (r, g, b, alpha_scaling))
    working_surface.unlock()
    return working_surface
