from lostsky.worldmap.event import BattleEvent
from lostsky.battle.mapdata import MapData
from lostsky.battle.mapaction import MapActionEvent, TurnNumTrigger, ArrivalTrigger, UnitAliveTrigger

class Mission(BattleEvent):

    def __init__(self):
        # Event Data
        name = 'Lost Sky Crisis'
        location = 'Graveyard'
        id_string = 'CH2ST3'
        prereqs = ['CH2ST2']
        auto_signup = True
        cancellable = False
        show_rewards = False
        desc = "Lately, there have been sightings of a wayward Kodama that hangs around the village graveyard. There are some rumors among the villagers that the Kodama are looking for some relic that's hidden away there."

        BattleEvent.__init__(self, name, location,
                             id_string, prereqs,
                             auto_signup, cancellable,
                             show_rewards, desc)

        # Map Data
        map_name = 'ch2st3v3.txt'
        mission_type = 'battle'
        objective = {'type':'Defeat Boss',
                     'target':'Miu',
                     'desc':'Defeat Miu to escape!'
                     }

        deploy_data = {'enable':True,
                       'max_units':8,
                       'preset_units':{},
                       'boxes':[(10, 9, 5, 2)]
                       }
        reward_list = [('spell_action', 'Rice Cake'),
                       ('treasure', 'synth_fire'),
                       ('treasure', 'synth_water'),
                       ('treasure', 'synth_earth')
                       ]

        # Enemy Unit Data
        enemy_unit_data = [{'template_name': 'Fuyuhana',
                                'unit_name': 'Fuyuhana',
                                    'level': 30},
                           # Kodama Lord Kotone's group
                           {'template_name': 'Kotone',
                                'unit_name': 'Kotone',
                                    'level': 8},
                           {'template_name': 'Healer Fairy',
                                'unit_name': 'Healer Fairy A',
                                    'level': 5},
                           {'template_name': 'Firefly',
                            'unit_name': 'Firefly A',
                            'level': 6},
                           {'template_name': 'Firefly',
                            'unit_name': 'Firefly B',
                            'level': 6},

                           {'template_name': 'Firefly',
                            'unit_name': 'Firefly C',
                            'level': 6},
                           {'template_name': 'Firefly',
                            'unit_name': 'Firefly D',
                            'level': 6},

                           # Kodama Lord Ayaka's group
                           {'template_name': 'Ayaka',
                                'unit_name': 'Ayaka',
                                    'level': 13},
                           {'template_name': 'Walking Tree',
                            'unit_name': 'Cursed Tree A',
                            'level': 12},
                           {'template_name': 'Walking Tree',
                            'unit_name': 'Cursed Tree B',
                            'level': 12},
                           {'template_name': 'Walking Tree',
                            'unit_name': 'Cursed Tree C',
                            'level': 12},
                           {'template_name': 'Walking Tree',
                            'unit_name': 'Cursed Tree D',
                            'level': 12},
                           {'template_name': 'Walking Tree',
                            'unit_name': 'Cursed Tree E',
                            'level': 12},



                           # Kodama Lord Haruna's Group
                           {'template_name': 'Haruna',
                                'unit_name': 'Haruna',
                                    'level': 8},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy A',
                                    'level': 5},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy B',
                                    'level': 5},

                           # Central Island Group
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree A',
                                    'level': 5},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree B',
                                    'level': 5},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree C',
                                    'level': 5},

                           # Kodama Lord Miu's exit guard group
                           {'template_name': 'Miu',
                                'unit_name': 'Miu',
                                    'level': 8},
                           {'template_name': 'Walking Tree',
                                'unit_name': 'Walking Tree D',
                                    'level': 6},
                           {'template_name': 'Healer Fairy',
                                'unit_name': 'Healer Fairy B',
                                    'level': 6},

                           # Reinforcement group
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy C',
                                    'level': 6},
                           {'template_name': 'Fairy',
                                'unit_name': 'Fairy D',
                                    'level': 6},

                          ]

        initial_spells = {'Fuyuhana':['Leaf Crystal'],
                          'Kotone':['Fireball'],
                          'Ayaka':['Dagger Throw'],
                          'Haruna':['Holy Amulet'],
                          'Miu':['Holy Amulet'],

                          'Walking Tree A':['Leaf Crystal'],
                          'Walking Tree B':['Leaf Crystal'],
                          'Walking Tree C':['Leaf Crystal'],
                          'Walking Tree D':['Leaf Crystal'],


                          'Fairy A':['Fireball'],
                          'Fairy B':['Fireball'],
                          'Cursed Tree A':['Poison Dust'],
                          'Cursed Tree B':['Poison Dust'],
                          'Cursed Tree C':['Poison Dust'],
                          'Cursed Tree D':['Leaf Crystal'],
                          'Cursed Tree E':['Leaf Crystal'],

                          'Fairy C':['Dagger Throw'],
                          'Fairy D':['Dagger Throw'],

                          'Healer Fairy A':['Healing Drop'],
                          'Healer Fairy B':['Healing Drop'],

                          'Firefly A':['Fireball'],
                          'Firefly B':['Poison Dust'],
                          'Firefly C':['Poison Dust'],
                          'Firefly D':['Fireball']

                          }
        initial_traits = {'Healer Fairy A':['Move+ Lv.1', 'Flight'],
                          'Healer Fairy B':['Move+ Lv.1', 'Flight'],
                          'Firefly A':['Flight'],
                          'Firefly B':['Flight'],
                          'Firefly C':['Flight'],
                          'Firefly D':['Flight'],
                          'Cursed Tree A':['Move+ Lv.2', 'Mirage'],
                          'Cursed Tree B':['Move+ Lv.2', 'Mirage'],
                          'Cursed Tree C':['Move+ Lv.2', 'Mirage'],
                          'Cursed Tree D':['Move+ Lv.2', 'Mirage'],
                          'Cursed Tree E':['Move+ Lv.2', 'Mirage'],


                          'Ayaka':['Mirage', 'Agility+ Lv.3'],
                          'Haruna':['Danmaku Sniper', 'Tactician'],
                          'Miu':['Flight']
                          }
        initial_ai_states = {'Kotone':'Defend',
                             'Ayaka':'Defend',
                             'Haruna':'Defend',
                             'Miu':'Defend',

                             'Walking Tree A':'Defend',
                             'Walking Tree B':'Defend',
                             'Walking Tree C':'Attack',
                             'Walking Tree D':'Attack',

                             'Fairy A':'Defend',
                             'Fairy B':'Defend',
                             'Cursed Tree A':'Pursuit',
                             'Cursed Tree B':'Pursuit',
                             'Cursed Tree C':'Pursuit',
                             'Cursed Tree D':'Pursuit',
                             'Cursed Tree E':'Pursuit',
                             'Fairy C':'Attack',
                             'Fairy D':'Attack',

                             'Firefly A':'Defend',
                             'Firefly B':'Defend',
                             'Firefly C':'Defend',
                             'Firefly D':'Defend',

                             'Healer Fairy A':'HealerStandby',
                             'Healer Fairy B':'HealerStandby'

                             }
        initial_locations = {'Keine':(-1, -1),
                             'Youmu':(-1, -1),
                             'Ran':(-1, -1),
                             'Chen':(-1, -1),
                             'Reimu':(-1, -1),
                             'Marisa':(-1, -1),
                             'Ayaka':(12, 5)

                             }
        reserve_units = ['Fuyuhana',
                         'Kotone',
                         'Haruna',
                         'Miu',
                         'Cursed Tree D',
                         'Cursed Tree E',
                         'Walking Tree A',
                         'Walking Tree B',
                         'Walking Tree C',
                         'Walking Tree D',
                         'Fairy A',
                         'Fairy B',
                         'Cursed Tree A',
                         'Cursed Tree B',
                         'Cursed Tree C',
                         'Fairy C',
                         'Fairy D',
                         'Firefly A',
                         'Firefly B',
                         'Firefly C',
                         'Firefly D',
                         'Healer Fairy A',
                         'Healer Fairy B',
                         ]


        all_landmarks = [{'name':'GS1',
                          'id_string':'tombstone',
                          'location':(4, 10)},
                          {'name':'GS2',
                          'id_string':'tombstone',
                          'location':(6, 10)},
                          {'name':'GS3',
                          'id_string':'tombstone',
                          'location':(6, 8)},
                          {'name':'GS4',
                          'id_string':'tombstone',
                          'location':(4, 8)},
                          {'name':'GS5',
                          'id_string':'tombstone',
                          'location':(4, 6)},
                          {'name':'GS6',
                          'id_string':'tombstone',
                          'location':(6, 6)},
                          {'name':'GS7',
                          'id_string':'tombstone',
                          'location':(11, 6)},
                          {'name':'GS8',
                          'id_string':'tombstone',
                          'location':(13, 6)},
                          {'name':'GS9',
                          'id_string':'tombstone',
                          'location':(11, 8)},
                          {'name':'GS10',
                          'id_string':'tombstone',
                          'location':(13, 8)},
                          {'name':'GS11',
                          'id_string':'tombstone',
                          'location':(11, 10)},
                          {'name':'GS12',
                          'id_string':'tombstone',
                          'location':(13, 10)},
                          {'name':'GS13',
                          'id_string':'tombstone',
                          'location':(18, 6)},
                          {'name':'GS14',
                          'id_string':'tombstone',
                          'location':(20, 6)},
                          {'name':'GS15',
                          'id_string':'tombstone',
                          'location':(18, 8)},
                          {'name':'GS16',
                          'id_string':'tombstone',
                          'location':(20, 8)},
                          {'name':'GS17',
                          'id_string':'tombstone',
                          'location':(18, 10)},
                          {'name':'GS18',
                          'id_string':'tombstone',
                          'location':(20, 10)},

                          {'name':'Mini Shrine',
                          'id_string':'minishrine',
                          'location':(12, 3)},
                          {'name':'CB0',
                          'id_string':'cherryblossom_tree',
                          'location':(11, 3)},
                          {'name':'CB1',
                          'id_string':'cherryblossom_tree',
                          'location':(13, 3)},

                          {'name':'CB2',
                          'id_string':'cherryblossom_tree',
                          'location':(9, 25)},
                          {'name':'CB3',
                          'id_string':'cherryblossom_tree',
                          'location':(14, 25)},

                          {'name':'Torii',
                          'id_string':'small_torii',
                          'location':(12, 11)},
                         ]

        required_starters = ['Youmu', 'Chen', 'Ran', 'Marisa', 'Reimu', 'Keine']
        pre_mission_MAE = PreMissionMAE()
        mid_mission_MAE_list = [WestTreasureMAE(), EastTreasureMAE(), MiuReinforceMAE(),
                                AyakaHarunaMAE()]
        required_survivors = ['Youmu', 'Ran', 'Chen', 'Marisa', 'Reimu', 'Keine',
                              'Mokou', 'Kotone', 'Miu', 'Haruna', 'Ayaka']
        post_mission_MAE = PostMissionMAE()

        self.map_data = MapData(map_name, mission_type, objective,
                                deploy_data, reward_list, enemy_unit_data,
                                initial_spells, initial_traits, initial_ai_states,
                                initial_locations, reserve_units, all_landmarks,
                                required_starters, pre_mission_MAE, mid_mission_MAE_list,
                                required_survivors, post_mission_MAE)

class PreMissionMAE(MapActionEvent):

    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self, triggers)

    def execute(self):
        """
        Big encounter with Fuyuhana
        """
        self.set_cursor_state(False)
        self.set_stats_display(False)

        self.play_music('battle04')

        self.add_to_party('Mokou')
        self.assign_spell('Mokou', 'Dagger Throw')
        self.assign_spell('Mokou', 'Flying Phoenix')
        self.assign_spell('Mokou', 'Rice Cake')
        self.set_unit_pos('Mokou', (12, 9))

        self.center_on('Mokou')

        self.say("Kodama Lords...? There haven't been Kodama Lords in Gensokyo for generations. What's the meaning of this?",
            'Mokou',
            'Mokou')

        self.say("Now that we've awakened, you'll be seeing a lot more of us.",
            'Ayaka',
            'Ayaka')

        self.say("Fujiwara no Mokou, you're no ordinary human are you? Ah... I see. The same flame burns in you that tints my leaves a brilliant red.",
            'Voice',
            None)

        self.say("We're ready to bind you to the sacred branch that's enshrined here!",
            'Ayaka',
            'Ayaka')

        self.say("Then let the summer sun herald the return of the Kodama Lords!",
            'Voice',
            None)

        self.fade_to_color('white', 1)
        self.deploy_unit('Fuyuhana', (12, 4))
        self.fade_from_color('white', 1)

        self.say("My lady, it has been too long. Your loyal servant, Ayaka of the Dawning Fog, is ready to serve you once more!",
            'Ayaka',
            'Ayaka')
        self.say("Living as a phantom was rather restricting. Now then, Ayaka... keep her busy while I conjure the cloud gathering spell.",
            'Fuyuhana',
            'Fuyuhana')
        self.say("I don't know what your goals are, but I have a precious friend in the village and I'm not going to let her come to harm.",
            'Mokou',
            'Mokou')

        self.move_unit('Mokou', (12, 5))
        self.fade_to_color('white', 0.2)
        self.play_sfx('miss')
        self.set_unit_pos('Ayaka', (12, 10))
        self.fade_from_color('white', 0.2)

        self.emote('Mokou', 'questionmark')

        self.move_unit('Mokou', (12, 10))
        self.fade_to_color('white', 0.2)
        self.play_sfx('miss')
        self.set_unit_pos('Ayaka', (12, 8))
        self.fade_from_color('white', 0.2)

        self.emote('Mokou', 'scribble')

        self.say("What's wrong with my aim today? I can't seem to hit her...",
            'Mokou',
            'Mokou')
        self.say("Hmm....",
            'Ayaka',
            'Ayaka')

        self.set_unit_pos('Youmu', (15, 16))
        self.set_unit_pos('Ran', (15, 16))
        self.set_unit_pos('Chen', (15, 16))

        self.set_unit_pos('Marisa', (9, 16))
        self.set_unit_pos('Reimu', (9, 16))
        self.set_unit_pos('Keine', (9, 16))

        self.move_unit('Youmu', (15, 11))
        self.move_unit('Ran', (14, 12))
        self.move_unit('Chen', (16, 12))

        self.move_unit('Marisa', (9, 11))
        self.move_unit('Reimu', (8, 12))
        self.move_unit('Keine', (10, 12))


        self.say("Mokou!",
            'Keine',
            'Keine')

        self.say("You guys are here? I heard about the commotion in the village and came to investigate.",
            'Mokou',
            'Mokou')

        self.say("You're the culprits behind this crisis!",
            'Youmu',
            'Youmu')

        self.set_invincibility_state('Ayaka', True)
        self.move_unit('Youmu', (13, 8))

        temp_spell_action = self.map.all_units_by_name['Youmu'].spell_actions[0]

        self.map.all_units_by_name['Youmu'].spell_actions[0]  = None
        self.assign_spell('Youmu', "Dagger Throw")
        self.script_battle('Youmu', 'Ayaka', {'rhs_equip':0, 'lhs_equip':0, 'lhs_hit':False, 'rhs_hit':True, 'lhs_crit':False, 'rhs_crit':False})
        self.map.all_units_by_name['Youmu'].spell_actions[0]  =  temp_spell_action

        self.emote('Youmu', 'scribble')


        self.say("Let me give it a shot!",
            'Marisa',
            'Marisa')

        self.move_unit('Marisa', (9, 8))

        temp_spell_action = self.map.all_units_by_name['Marisa'].spell_actions[0]

        self.set_spirit_charge('Marisa', 500)
        self.map.all_units_by_name['Marisa'].spell_actions[0]  = None
        self.assign_spell('Marisa', "Master Spark")
        self.script_battle('Marisa', 'Ayaka', {'rhs_equip':0, 'lhs_equip':0, 'lhs_hit':False, 'rhs_hit':False, 'lhs_crit':False, 'rhs_crit':False})
        self.map.all_units_by_name['Marisa'].spell_actions[0]  =  temp_spell_action

        self.emote('Marisa', 'questionmark')



        self.say("Huh? I missed even with a Master Spark?",
            'Marisa',
            'Marisa')

        self.say("She dodged so effortlessly.",
            'Youmu',
            'Youmu')

        self.say("Thank you for the distraction, Ayaka. The spell is ready.",
            'Fuyuhana',
            'Fuyuhana')

        self.fade_to_color('white', 0.5)
        self.show_image('clouds001', 'sky001.jpg', (0, 0))
        self.fade_from_color('white', 0.5)


        self.say("Ah! Look up in the sky!",
            'Chen',
            'Chen')

        # Show image of sky with clouds -> sky without clouds
        self.fade_to_color('white', 0.5)
        self.hide_image('clouds001')
        self.show_image('clouds002', 'sky002.jpg', (0, 0))
        self.fade_from_color('white', 0.5)

        self.say("The clouds... they've all disappeared!",
            'Keine',
            'Keine')

        self.fade_to_color('white', 0.5)
        self.hide_image('clouds002')
        self.fade_from_color('white', 0.5)

        self.say("She intends to steal Gensokyo's rainfall?",
            'Ran',
            'Ran')

        self.say("We Kodama Lords have long been protectors of the forest, but after centuries of our absence, it's clear that the land... and Gensokyo as a whole, is in need of our leadership again.",
            'Fuyuhana',
            'Fuyuhana')
        self.say(" Oh, please. We don't need you high and mighty lords ruling over us.",
            'Marisa',
            'Marisa')
        self.say("I am returning to the sanctuary. Ayaka, I expect you to deal them a crushing defeat.",
            'Fuyuhana',
            'Fuyuhana')
        self.say("Of course!",
            'Ayaka',
            'Ayaka')




        # Ayaka's Party Deploys
        self.move_unit('Ayaka', (12, 3))

        self.fade_to_color('white', 0.5)
        self.kill_unit('Fuyuhana')
        self.deploy_unit('Cursed Tree A', (12, 1))
        self.deploy_unit('Cursed Tree B', (10, 1))
        self.deploy_unit('Cursed Tree C', (14, 1))
        self.deploy_unit('Cursed Tree D', (11, 2))
        self.deploy_unit('Cursed Tree E', (13, 2))
        self.fade_from_color('white', 0.5)


        self.center_on('Ayaka')

        # Haruna's Party Deploys
        self.fade_to_color('white', 0.5)
        self.deploy_unit('Haruna', (5, 22))
        self.deploy_unit('Fairy A', (6, 23))
        self.deploy_unit('Fairy B', (4, 23))
        self.deploy_unit('Walking Tree A', (3, 21))
        self.deploy_unit('Walking Tree B', (7, 21))
        self.fade_from_color('white', 0.5)

        self.center_on('Haruna')
        self.emote('Haruna', 'zzz')
        self.say("Haruna!",
            'Ayaka',
            'Ayaka')
        self.say("*Yawn* Did we have to return so early?",
            'Haruna',
            'Haruna')

        # Kotone's Group
        self.fade_to_color('white', 0.5)
        self.deploy_unit('Kotone', (18, 24))
        self.deploy_unit('Healer Fairy A', (17, 24))
        self.deploy_unit('Firefly A', (20, 19))
        self.deploy_unit('Firefly B', (15, 19))
        self.deploy_unit('Firefly C', (19, 25))
        self.deploy_unit('Firefly D', (16, 25))
        self.fade_from_color('white', 0.5)
        self.center_on('Kotone')
        self.startle('Kotone')
        self.say("Ayaka! I, Kodama Sorceress Kotone, am ready for battle! Oh! I see Lady Ayaka has brought her prized Cursed Trees.",
            'Kotone',
            'Kotone')


        # Miu's Group
        self.fade_to_color('white', 0.5)
        self.deploy_unit('Miu', (13, 32))
        self.deploy_unit('Healer Fairy B', (13, 33))
        self.deploy_unit('Walking Tree C', (10, 33))
        self.deploy_unit('Walking Tree D', (16, 33))
        self.fade_from_color('white', 0.5)

        self.center_on('Miu')
        self.say("I'm here too! Ah... ready to receive orders!",
            'Miu',
            'Miu')

        self.center_on('Youmu')

        self.say("Four of them? We'll gladly send them all packing.",
            'Marisa',
            'Marisa')
        self.say("I've analyzed Ayaka's spiritual energy. She's protected by a magical mist barrier. Any spells we shoot at it will pass right through to the other side without harming her.",
            'Ran',
            'Ran')
        self.emote('Marisa', 'annoyed')
        self.say("That's cheating!",
            'Marisa',
            'Marisa')
        self.say("I have a feeling these Kodama Lords won't play along with our one-on-one duels that we've been having.",
            'Reimu',
            'Reimu')
        self.say("Ran, can you tell if the others are protected as well?",
            'Mokou',
            'Mokou')
        self.say("I don't sense the same magic around them. They should be vulnerable to our usual attacks.",
            'Ran',
            'Ran')
        self.say("That one furthest out seems to be the least confident of herself and one of them is half-asleep.",
            'Reimu',
            'Reimu')
        self.say("Let's... retreat to the human village for now.",
            'Youmu',
            'Youmu')
        self.emote('Marisa', 'scribble')
        self.say("We're going to turn tail and run?",
            'Marisa',
            'Marisa')
        self.say("Cheer up, Marisa. We'll get our decisive battle in due time.",
            'Reimu',
            'Reimu')
        self.say("We can't win against Ayaka at the moment, but Akyu may know a way. After all, the Kodama Lords were defeated once in the past.",
            'Keine',
            'Keine')




        self.set_cursor_state(True)
        self.set_stats_display(True)


class WestTreasureMAE(MapActionEvent):

    def __init__(self):
        triggers = [ArrivalTrigger((9, 25, 1, 1), 1)]
        MapActionEvent.__init__(self, triggers)

    def execute(self):
        self.say("Buried at the foot of the cherry tree is a small rusted box.",
                None,
                None)
        self.say("Acquired Treasure Item: Lava Rock!",
                None,
                None)
        self.add_item('treasure', 'synth_fire', 1)


class EastTreasureMAE(MapActionEvent):

    def __init__(self):
        triggers = [ArrivalTrigger((14, 25, 1, 1), 1)]
        MapActionEvent.__init__(self, triggers)

    def execute(self):
        self.say("Sitting at the side of the cherry tree is a cardboard box.",
                None,
                None)
        self.say("Acquired Treasure Item: Cell Phone!",
                None,
                None)
        self.add_item('treasure', '002_cellphone', 1)


class MiuReinforceMAE(MapActionEvent):

    def __init__(self):
        triggers = [TurnNumTrigger(3),
                    # Ayaka must be still alive
                    UnitAliveTrigger('Ayaka', True)]
        MapActionEvent.__init__(self, triggers)

    def execute(self):
        """
        Ayaka sends reinforcements to back up Miu
        """
        self.set_stats_display(False)
        self.center_on('Ayaka')

        self.say("Miu, they're coming your way! Get ready!",
                'Ayaka',
                'Ayaka')

        self.center_on('Miu')
        self.say("Got it!",
                'Miu',
                'Miu')
        self.center_on('Ayaka')
        self.say("I'm sending some of our fairies to assist you. These guys are fast, but we'll catch up to them soon.",
                'Ayaka',
                'Ayaka')

        self.center_on('Miu')
        self.deploy_unit('Fairy D', (13, 34))
        self.move_unit('Fairy D', (11, 33))
        self.deploy_unit('Fairy C', (13, 34))
        self.move_unit('Fairy C', (15, 33))

        self.say("We'll not fail you, Lady Ayaka!",
                'Fairy',
                'Fairy')


        self.say("I have to hand it to them. They're more organized than any troublemakers we've encountered in the past.",
                'Reimu',
                'Reimu')


        self.set_stats_display(True)


class AyakaHarunaMAE(MapActionEvent):

    def __init__(self):
        triggers = [TurnNumTrigger(5),
                    UnitAliveTrigger('Ayaka', True),
                    UnitAliveTrigger('Haruna', True)
                    ]
        MapActionEvent.__init__(self, triggers)

    def execute(self):
        """
        Ayaka + Haruna converse about old times
        """
        self.set_stats_display(False)
        self.center_on('Ayaka')
        self.say("Are you okay, Ayaka?",
                'Haruna',
                'Haruna')
        self.say("As best as can be. They're tough, but we've got the strength in numbers.",
                'Ayaka',
                'Ayaka')
        self.center_on('Haruna')
        self.say("You know, it's been long time since we've seen battle. A lot has changed in this world.",
                'Haruna',
                'Haruna')
        self.center_on('Ayaka')
        self.say("Miu and Kotone wouldn't know it. Our world is a lot different from when we went to sleep. Many new youkai have shown up.",
                'Ayaka',
                'Ayaka')
        self.center_on('Haruna')
        self.say("Yeah... we've been cut off from the outside world. I wonder if we'll ever see our old companions again.",
                'Haruna',
                'Haruna')
        self.set_stats_display(True)


class AyakaYoumuMAE(MapActionEvent):

    def __init__(self):
        triggers = [TurnNumTrigger(7),
                    UnitAliveTrigger('Ayaka', True),
                    UnitAliveTrigger('Youmu', True),
                    UnitAliveTrigger('Chen', True)
                    ]
        MapActionEvent.__init__(self, triggers)

    def execute(self):
        """
        Ayaka talks with Youmu during their fight
        """
        self.set_stats_display(False)
        self.center_on('Ayaka')
        self.say("There you are!",
                'Ayaka',
                'Ayaka')
        self.center_on('Chen')
        self.say("Ah! These guys are tough.",
                'Chen',
                'Chen')
        self.center_on('Youmu')
        self.say("Hurry! We need to keep moving!",
                'Youmu',
                'Youmu')
        self.set_stats_display(True)

class PostMissionMAE(MapActionEvent):
    def __init__(self):
        triggers = []
        MapActionEvent.__init__(self, triggers)

    def pre_exec(self):

        self.set_cursor_state(False)
        self.set_stats_display(False)
        self.remove_all_enemies()
        self.set_unit_pos('Youmu', (13, 34))
        self.set_unit_pos('Ran', (12, 33))
        self.set_unit_pos('Chen', (12, 32))
        self.set_unit_pos('Reimu', (13, 33))
        self.set_unit_pos('Marisa', (13, 32))
        self.set_unit_pos('Mokou', (14, 33))
        self.set_unit_pos('Keine', (14, 32))


        self.set_unit_pos('Ayaka', (15, 8))
        self.set_unit_pos('Haruna', (9, 8))
        self.set_unit_pos('Miu', (12, 10))
        self.set_unit_pos('Kotone', (12, 6))

    def execute(self):

        # Heroes escape from the scene
        self.center_on('Youmu')
        self.say("We made it!",
                'Youmu',
                'Youmu')
        self.say("Everyone all right?",
                'Keine',
                'Keine')
        self.say("A few bruises but I'm okay.",
                'Mokou',
                'Mokou')
        self.say("Let's hurry and get out of here then.",
                'Marisa',
                'Marisa')

        self.move_unit('Youmu', (13, 36))
        self.move_unit('Ran', (13, 36))
        self.move_unit('Chen', (13, 36))
        self.move_unit('Marisa', (13, 36))
        self.move_unit('Reimu', (13, 36))
        self.move_unit('Mokou', (13, 36))
        self.move_unit('Keine', (13, 36))

        # Kodana Lords talk about their defeat
        self.center_on('Haruna')

        self.say("My apologies, Ayaka. We let them escape.",
                'Haruna',
                'Haruna')
        self.say("Miu, we should have supported you more in that battle. If only we had more time to train you.",
                'Kotone',
                'Kotone')
        self.say("It's my fault. I failed to fulfill my duty to the forest.",
                'Miu',
                'Miu')
        self.emote('Kotone', 'annoyed')
        self.move_unit('Haruna', (12, 8))
        self.say("That's enough. You both fought hard and bravely. We can't have everything go our way.",
                'Haruna',
                'Haruna')
        self.startle('Kotone')
        self.say("Let me chase after them!",
            'Kotone',
            'Kotone')
        self.say("You're exhausted, Kotone. What good will charging alone into their stronghold do?",
            'Haruna',
            'Haruna')
        self.say("Haruna is right. Having all just awakened, we're at our weakest. Return to the forest and ensure that nothing goes wrong with the cloud gathering.",
            'Ayaka',
            'Ayaka')
        self.say("Until Fuyuhana completes the rain making ceremony to strengthen the forest, it remains in danger so long as we are away from it.",
            'Haruna',
            'Haruna')

        self.emote('Kotone', 'exclamation')
        self.emote('Miu', 'exclamation')
        self.say("At once!",
                'Miu and Kotone',
                None)

        self.move_unit('Kotone', (12, -1))
        self.move_unit('Haruna', (13, 8))
        self.move_unit('Miu', (12, -1))
        self.kill_unit('Kotone')
        self.kill_unit('Miu')

        self.say("I can see that working with these new Kodama Lords will be difficult.",
                'Ayaka',
                'Ayaka')
        self.say("As your mentor for so long, all I can say is to put your trust in them. We were in the same place at one time.",
                'Haruna',
                'Haruna')
        self.say("I suppose you're right.",
                'Ayaka',
                'Ayaka')

        self.say("Haruna, I have an assignment. You are the most capable of swaying trees to fight on our side.",
                'Ayaka',
                'Ayaka')
        self.emote('Haruna', 'questionmark')
        self.say("Fuyuhana wants to make sure that that sacred sanctuary on Youkai Mountain is safe. Don't let the Tengu... or those heroes... near it.",
                'Ayaka',
                'Ayaka')
        self.say("I see. I'll set out with a handful of fairies immediately.",
                'Haruna',
                'Haruna')
        self.say("Best of luck, Haruna.",
                'Ayaka',
                'Ayaka')

        self.stop_music()
        self.set_stats_display(True)
        self.set_cursor_state(False)
